module ToClause(
   Literal(..),
   toClause, 
   rewriteNeg, rewriteImpEquiv, surfConj, 
   formulaToClause,negateConjecture,
   clauseToString,
   clauseSetToString) 
where
import PCFormula
import Debug.Trace

data Literal = Lit String
             | NegLit String 
	     deriving (Eq,Read,Show)

type Clause = [Literal]
 
formulaToClause xs =  (toClause . rewriteNeg . rewriteImpEquiv . parse) xs


rewriteImpEquiv (Var s) = (Var s)
rewriteImpEquiv (Neg f) = (Neg (rewriteImpEquiv f))
rewriteImpEquiv (Disj f0 f1) = Disj (rewriteImpEquiv f0) (rewriteImpEquiv f1)
rewriteImpEquiv (Conj f0 f1) = Conj (rewriteImpEquiv f0) (rewriteImpEquiv f1)
rewriteImpEquiv (Imp f0 f1) = Disj (Neg (rewriteImpEquiv f0))
		                   (rewriteImpEquiv f1)
rewriteImpEquiv (Equiv f0 f1) = 
      let 
        rf0 = rewriteImpEquiv f0 
        rf1 = rewriteImpEquiv f1
      in Conj (Disj (Neg rf0) rf1) (Disj (Neg rf1) (rf0))

rewriteNeg (Var s) = (Var s)
rewriteNeg (Neg (Var s)) = (Neg (Var s))
rewriteNeg (Neg (Neg f)) = rewriteNeg f
rewriteNeg (Neg (Disj f0 f1)) = Conj (rewriteNeg (Neg (rewriteNeg f0)))
                                     (rewriteNeg (Neg (rewriteNeg f1)))
rewriteNeg (Neg (Conj f0 f1)) = Disj (rewriteNeg (Neg (rewriteNeg f0)))
                                     (rewriteNeg (Neg (rewriteNeg f1)))
rewriteNeg (Disj f0 f1) = Disj (rewriteNeg f0) 
                               (rewriteNeg f1)
rewriteNeg (Conj f0 f1) = Conj (rewriteNeg f0) 
                               (rewriteNeg f1)




surfConj (Disj (Conj a0 a1) a2) = Conj (surfConj (Disj a0 a2))
                                       (surfConj (Disj a1 a2))
surfConj (Disj a0 (Conj a1 a2)) = Conj (surfConj (Disj a0 a1))
                                       (surfConj (Disj a0 a2))
surfConj (Disj a0 a1) = 
   let 
     sa0 = surfConj a0
     sa1 = surfConj a1
   in 
     if (isConj sa0) || (isConj sa1)
     then surfConj (Disj sa0 sa1)
     else Disj sa0 sa1
surfConj (Conj a0 a1) = Conj (surfConj a0) (surfConj a1)
surfConj (Neg (Var a0)) = (Neg (Var a0))
surfConj (Var a0) = (Var a0)
surfConj a0 = error ("surfConj:"++show a0)

isConj (Conj _ _) = True
isConj _ = False


flattenConj (Conj a0 a1) = (flattenConj a0)++(flattenConj a1)
flattenConj (Disj a0 a1) = [(flattenDisj a0)++(flattenDisj a1)]
flattenConj (Neg (Var x)) = [[NegLit x]]
flattenConj (Var x) = [[Lit x]]

flattenDisj (Disj a0 a1) = (flattenDisj a0)++(flattenDisj a1)
flattenDisj (Neg (Var x)) = [NegLit x]
flattenDisj (Var x) = [Lit x]
flattenDisj x = error (show x)


toClause  = flattenConj . surfConj 

negateFormula f = (Neg f)

negateConjecture  = toClause . rewriteNeg . rewriteImpEquiv . negateFormula . parse

clauseToString [] = ""
clauseToString [(Lit x)] = x
clauseToString [(NegLit x)] = ('-':x)
clauseToString ((Lit x0):x1:xs) = x0++" | "++(clauseToString (x1:xs))
clauseToString ((NegLit x0):x1:xs) = ('-':x0)++" | "++(clauseToString (x1:xs))
   
clauseSetToString clauses = (foldr (\ x y->  x++"\n"++y) 
                                   [] 
                                   (map clauseToString clauses)) 
